import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_story/src/repository/story.dart';

///
/// Events
///
abstract class StoryUpdateEvent {}

class StoryUpdateSeenEvent extends StoryUpdateEvent {
  final String storyId;

  StoryUpdateSeenEvent(this.storyId);
}

class StoryUpdateDeleteEvent extends StoryUpdateEvent {
  final String storyId;

  StoryUpdateDeleteEvent(this.storyId);
}

// Story
abstract class StoryUpdateState {
  final StoryUpdateEvent event;
  StoryUpdateState(this.event);
}

class StoryUpdateSuccessState extends StoryUpdateState {
  StoryUpdateSuccessState(StoryUpdateEvent event) : super(event);
}

class StoryUpdateInitialState extends StoryUpdateState {
  StoryUpdateInitialState() : super(null);
}

class StoryUpdateLoadingState extends StoryUpdateState {
  StoryUpdateLoadingState(StoryUpdateEvent event) : super(event);
}

class StoryUpdateErrorState extends StoryUpdateState {
  final String error;
  StoryUpdateErrorState(StoryUpdateEvent event, this.error) : super(event);
}

class StoryUpdateBloc extends Bloc<StoryUpdateEvent, StoryUpdateState> {
  final StoryRepository storyRepository;
  StoryUpdateBloc(this.storyRepository) : super(StoryUpdateInitialState());

  @override
  Stream<StoryUpdateState> mapEventToState(StoryUpdateEvent event) async* {
    AppResponse<bool> response;
    yield StoryUpdateLoadingState(event);
    if (event is StoryUpdateSeenEvent) {
      response = await storyRepository.addStorySeen(event.storyId);
    }
    if (event is StoryUpdateDeleteEvent) {
      response = await storyRepository.deleteStory(event.storyId);
    }
    if (response != null) {
      if (response.isSuccess) {
        yield StoryUpdateSuccessState(event);
      } else {
        yield StoryUpdateErrorState(event, response.error);
      }
    }
  }
}
